// -*- mode: C++ -*- 
//
// Copyright (C) 2006 Christian Holm Christensen <cholm@nbi.dk>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2.1
// of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
// 02111-1307 USA
//
/** @file    FXS.h
    @author  Christian Holm Christensen <cholm@nbi.dk>
    @date    Sat Jul  8 01:42:16 2006
    @brief   File eXchange Server interface 
    @ingroup fmdda_data    
*/
#ifndef FMDDA_FXS
#define FMDDA_FXS
#ifndef RCUDB_SQL
# include <rcudb/Sql.h>
#endif
#ifndef RCUDB_SERVER
# include <rcudb/Server.h>
#endif
#ifndef __SSTREAM__
# include <sstream>
#endif
#ifndef __IOMANIP__
# include <iomanip>
#endif

namespace FmdDA
{
  /** @class FXSSession 
      @brief A guard class to ensure that we leave the data base in a
      good state.  On construction, and entry is written to the data
      base.  If the user calls the member function Failure sometime
      between the contruction of an object of this class, and the
      destruction of the object, then, the database entry is removed. 

      @code 
      void Some::Scope() 
      { 
         FXSSesstion session(fServer, fRun, fDetector, fPath, size, 
	                     created, source, md5);
	 if (!session.IsSuccess()) return;

	 ...
         if (!ok) session.Failure(); // Indicate we should roll-back
	 
	 // Session object goes out of scope. If Failure was called, 
	 // then the automatic destruction of the session object
	 // ensures that we roll-back to the previous state. 
      }
      @endcode 

      This is especially useful with exceptions. 
   */
  struct FXSSession 
  {
    /** Construcor 
	@param server    Database server
	@param run       Run number
	@param detector  Detector code
	@param fileId    File identifier 
	@param path      Path of file on FXS
	@param size      Size of file 
	@param created   Date of creation
	@param source    Source of the file 
	@param md5       MD5 check sum of file */
    FXSSession(RcuDb& server, 
	       int run, 
	       const std::string& detector, 
	       const std::string& fileId, 
	       unsigned int       size, 
	       const std::string& created, 
	       const std::string& source, 
	       const std::string& md5)
      : fServer(server), 
	fDetector(detector), 
	fFileId(fileId), 
	fPath(path)
    {
      SetRun(run);
      SetPath(source);
      
      std::
      RcuDb::Sql sql;
      sql << "INSERT INTO daqFES_files(" 
	  << "run,"
	  << "detector,"
	  << "fileId,"
	  << "path,"
	  << "size,"
	  << "time_created,"
	  << "DAQsource,"
	  << "fileCheckSum"
	  << ") VALUES ("
	  << "'" << fRun      << "'" 
	  << "'" << fDetector << "'" 
	  << "'" << fFileId   << "'" 
	  << "'" << fPath     << "'" 
	  << "'" << size      << "'" 
	  << "'" << created   << "'" 
	  << "'" << fSource   << "'" 
	  << "'" << md5       << "')";
      fSuccess = server.Exec(sql);
    }
    /** Destructor.  If Failure was called, this will remove the entry
	inserted by the constructor */
    ~FXSSession() 
    {
      if (fSuccess) return;
      RcuDb::Sql sql;
      sql << "DELETE FROM daqFES_files WHERE " 
	  << "run='"      << fRun      << "' and "
	  << "detector='" << fDetector << "' and " 
	  << "fileId='"   << fFileId   << "' and " 
	  << "filePath='" << fPath     << "'";
      fServer.Exec(sql);
    }
    /** @return @c true if no error occured, or the user has called
	Failure  */
    bool IsSuccess() const { return fSuccess; }
    /** Indiate success */
    void Success() { fSuccess = true; }
    /** Indiate failure - causes a roll-back */
    void Failure() { fSuccess = false; }
  protected: 
    void SetRun(int run) 
    {
      std::stringstream srun;
      srun << std::setfill('0') << std::setw(9) << run;
      fRun = srun.str();
    }
    void SetPath(const std::string& source) 
    {
      std::stringstream spath;
      spath << "run" << fRun << "_" << fDetector << "_" 
	    << source << "_" << fFileId;
      fPath = spath.str();
    }
    
    /** Database server */
    RcuDb::Server& fServer;
    /** Run number */
    std::string    fRun;
    /** Detector string */
    std::string    fDetector;
    /** File identifer */
    std::string    fFileId;
    /** Path of file on FXS */
    std::string    fPath;
    /** Current status */
    bool           fSuccess;
  };
}

  
