// -*- mode: C++ -*- 
//
// Copyright (C) 2006 Christian Holm Christensen <cholm@nbi.dk>
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2.1
// of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
// 02111-1307 USA
//
/** @file    Maker.h
    @author  Christian Holm Christensen <cholm@nbi.dk>
    @date    Sat Jul  8 01:42:16 2006
    @brief   Utility class for various FMD analysers
    @ingroup fmdda_data    
*/
#ifndef FMDDA_MAKER
#define FMDDA_MAKER
#ifndef __VECTOR__
# include <vector>
#endif
#ifndef __STRING__
# include <string>
#endif
#ifndef __IOSFWD__
# include <iosfwd>
#endif
#ifndef FMDDA_AltroDecoder
# include <fmdda/AltroDecoder.h>
#endif
#ifndef FMDDA_ProgressMeter
# include <fmdda/ProgressMeter.h>
#endif

namespace FmdDA 
{
  // Forward decl 
  class Reader;
  
  /** @class Maker
      @brief Utility class for various FMD analysers
      @ingroup fmdda
  */
  class Maker : public AltroDecoder
  {
  public:
    /** List of sources */ 
    typedef std::vector<std::string> Sources_t;
    /** Constructor 
	@param src  List of source files to read from 
	@param out  Output file name 
	@param n    Max number of events, or all if < 0
	@param skip Number of events to skip from the beginning
	@param all  Read all or some events.
	@param wait Blocking read of events?
    */
    Maker(const Sources_t& src, 
	  const char*      out,
	  int              n=-1, 
	  unsigned int     skip=0, 
	  bool             all=true, 
	  bool             wait=false);

    /** Destructor */
    virtual ~Maker() {}

    /** Loop over the data */
    virtual bool Run();

    /** @param over Oversampling used when recording the data */
    void SetOverSample(unsigned int over) { fOverSample = over; }
    /** Set strip range 
	@param min Minimum strip 
	@param max Maximum strip */
    void SetRange(unsigned int min, unsigned int max) 
    {
      fMinStrip = min;
      fMaxStrip = max;
    }
    /** Set the offset 
	@param o offset. */
    void SetOffset(unsigned o=14) { fOffset = o; }
    /** Set the extra off set 
	@param e Extra offset */
    void SetExtraOffset(unsigned e=5) { fExtraOffset = e; }
  protected:
    /** Pre process the data collected */
    virtual bool Start() { return true; }
    /** Pre process the data collected */
    virtual bool Event();
    /** Post process the data collected */
    virtual bool End() = 0;
    /** @param t Time bin
	@return  Sub-sample corresponding to the time bin */
    unsigned int Timebin2Sample(unsigned t) const;
    /** @param t Time bin
	@return  strip corresponding to the time bin */
    unsigned int Timebin2Strip(unsigned t) const;
    /** @param t Time bin
	@return  strip and sub-sample corresponding to the time bin */
    float Timebin2StripSample(unsigned t) const;
    /** @param strip 
	@param sample 
	@return corresponding time bin */
    unsigned int StripSample2Timebin(unsigned strip, unsigned sample) const;
    
    /** Reader of raw data */
    Reader* fReader;
    /** Oversampling ratio used when recording the data */
    unsigned fOverSample;
    /** First strip */
    unsigned fMinStrip;
    /** Last strip */
    unsigned fMaxStrip;
    /** Offset */
    unsigned fOffset;
    /** Extra offset */
    unsigned fExtraOffset;
    /** Progress meteter */ 
    ProgressMeter fMeter;
    /** Name of output file */
    std::string            fOutput;
    /** Output stream */ 
    std::ostream*          fCsv;
  };
}

#endif
//
// EOF
//



